/*
  This file is under Public Domain.
*/

/*
  K-Meleon is built by Visual C++, then C++ classes in DLLs may not be use
  other compilers because of different name mangling rule.
  And I use small alternative class.
  If you remake this by VC, you should use usual mozilla classes.
 */

/*
  tiny alternative of nsCOMPtr
 */
template <class T>
struct miniCOMPtr {
	miniCOMPtr(T* ptr = nsnull) : m_ptr(ptr) { NS_IF_ADDREF(m_ptr); }
	~miniCOMPtr() { NS_IF_RELEASE(m_ptr); }
	operator bool() { return m_ptr; }
	operator T* () { return m_ptr; }
	T** operator&() { return &m_ptr; }
	T* operator->() { return m_ptr; }
	nsresult doQueryInterface(nsISupports* i);
	nsresult doGetService(const char* cid);
	template <class U> nsresult QueryInterface(miniCOMPtr<U>& i) {
		return i.doQueryInterface(m_ptr);
	}
private:
	T* m_ptr;
};

template <class T>
nsresult miniCOMPtr<T>::doQueryInterface(nsISupports* i) {
	nsresult rv;
	void* ptr;
	rv = i->QueryInterface(NS_GET_IID(T), &ptr);
	if (NS_SUCCEEDED(rv)) {
		NS_IF_RELEASE(m_ptr);
		m_ptr = reinterpret_cast<T*>(ptr);
	}
	return rv;
}

nsresult GetServiceByContractID(const char* cid, const nsIID& iid, void** result);

template <class T>
nsresult miniCOMPtr<T>::doGetService(const char* cid)
{
	nsresult rv;
	void* ptr;
	rv = GetServiceByContractID(cid, NS_GET_IID(T), &ptr);
	if (NS_SUCCEEDED(rv)) {
		NS_IF_RELEASE(m_ptr);
		m_ptr = reinterpret_cast<T*>(ptr);
	}
	return rv;
}
