/*
  This file is under public domain.
*/
#include <windows.h>
#include <stdlib.h>
#include <string.h>


char *utils_exist(char *name)
{
	if (name != NULL && 0xFFFFFFFF != GetFileAttributes(name)) {
		return _strdup("true");
	} else {
		return _strdup("false");
	}
}

void utils_rename(char *from, char *to)
{
	if (from != NULL && to != NULL) {
		MoveFile(from, to);
	}
}

void utils_copyfile(char *from, char *to)
{
	if (from != NULL && to != NULL) {
		CopyFile(from, to, TRUE);
	}
}

void utils_unlink(char *from, char *to)
{
	if (from != NULL && to != NULL) {
		DeleteFile(from);
	}
}

char *utils_read(char *name)
{
	HANDLE hFile;
	char *buffer = NULL;
	DWORD done;
	
	if (name != NULL) {
		hFile = CreateFile(name, GENERIC_READ, FILE_SHARE_READ, NULL,
		                   OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		if (hFile) {
			DWORD size, high;
			size= GetFileSize(hFile, &high);
			if (! high) {
				buffer = (char *) malloc(size + 1);
				if (buffer) {
					buffer[0] = '\0';
					ReadFile(hFile, buffer, size, &done, NULL);
					buffer[done] = '\0';
				}
			}
			CloseHandle(hFile);
		}
	}
	return buffer;
}

void utils_write(char *name, char *contents)
{
	HANDLE hFile;
	DWORD done;
	
	if (name != NULL) {
		hFile = CreateFile(name, GENERIC_WRITE, 0, NULL,
		                   CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
		if (hFile) {
			WriteFile(hFile, contents, strlen(contents), &done, NULL);
			CloseHandle(hFile);
		}
	}
}

void utils_append(char *name, char *contents)
{
	HANDLE hFile;
	DWORD done;
	
	if (name != NULL) {
		hFile = CreateFile(name, GENERIC_WRITE, 0, NULL,
		                   OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
		if (hFile) {
			SetFilePointer(hFile, 0, 0, FILE_END);
			WriteFile(hFile, contents, strlen(contents), &done, NULL);
			CloseHandle(hFile);
		}
	}
}


static BOOL create_directory(LPTSTR lpszPath, LPSECURITY_ATTRIBUTES lpsa)
{
	LPTSTR c;
	
	/* remove tail backslash */
	c = lpszPath + lstrlen(lpszPath) - 1;
	while (c > lpszPath && *c == TEXT('\\')) {
		*c = 0;
		c = CharPrev(lpszPath, c);
	}
	
	if (*lpszPath == 0)
		return FALSE;
	if (CreateDirectory(lpszPath, lpsa))
		return TRUE;
	if (GetLastError() != ERROR_PATH_NOT_FOUND)
		return FALSE;
	
	/* search last backslash */
	for (; *c != TEXT('\\'); c = CharPrev(lpszPath, c))
		if (c == lpszPath)
			return FALSE;
	
	*c = TEXT('\0');
	if (! create_directory(lpszPath, lpsa))
		return FALSE;
	*c = TEXT('\\');
	return CreateDirectory(lpszPath, lpsa);
}


static void slash2backslash(LPTSTR p)
{
	for (; *p != 0; p = CharNext(p))
		if (*p == TEXT('/')) *p = TEXT('\\');
}


char *utils_mkdir(char *name)
{
	if (name) {
		slash2backslash(name);
		if (create_directory(name, NULL))
			return _strdup("true");
	}
	return _strdup("false");
}


char *utils_slash2backslash(char *s)
{
	char *p;
	
	if (s == NULL)
		return _strdup("");
	p = _strdup(s);
	if (p) slash2backslash(p);
	return p;
}
