/*
  This file is under public domain.
*/

/*
  references
  
  seamonkey source
    netwerk/base/public/nsIPermissionManager.idl
    netwerk/base/public/nsIURI.idl
    netwerk/base/public/nsIIOService.idl
  
  Gecko SDK
    include\necko\nsIIOService.h
    include\necko\nsIPermissionManager.h
    include\necko\nsIURI.h
  
  K-Meleon source
    Permissions.h
    Permissions.cpp
  
  http://www.xulplanet.com/references/xpcomref/ifaces/nsIPermissionManager.html
  http://www.xulplanet.com/references/xpcomref/ifaces/nsIURI.html
 */

/*
  Debug Helper
  km.pluginmsg("utils", "HostDeny", "http://kmeleon.sf.net", "image")
  km.pluginmsg("utils", "HostAllow", "http://kmeleon.sf.net", "image")
  km.pluginmsg("utils", "HostRemove", "http://kmeleon.sf.net", "image")
  
  Known permission type strings are "cookie", "image", "popup".
 */

#include <windows.h>
#include <stdlib.h>
#include <string.h>
#define KMELEON_PLUGIN_EXPORTS
#include <kmeleon_plugin.h>

#include <nsNetCID.h>
#include <nsIIOService.h>
#include <nsIURI.h>
#include <nsEmbedString.h>
#include <nsIPermissionManager.h>
#include "miniCOMPtr.h"

static BOOL new_uri(const char *url, miniCOMPtr<nsIURI>& iuri)
{
	nsresult rv;
	
	miniCOMPtr<nsIIOService> ioserv;
	rv = ioserv.doGetService(NS_IOSERVICE_CONTRACTID);
	NS_ENSURE_SUCCESS(rv, FALSE);
	NS_ENSURE_TRUE(ioserv, FALSE);
	
	nsEmbedCString urlstr(url);
	
	rv = ioserv->NewURI(urlstr, NULL, NULL, &iuri);
	NS_ENSURE_SUCCESS(rv, FALSE);
	NS_ENSURE_TRUE(iuri, FALSE);
	
	return TRUE;
}


static BOOL addperm(char *url, char *type, PRUint32 perm)
{
	nsresult rv;
	
	miniCOMPtr<nsIPermissionManager> pman;
	rv = pman.doGetService(NS_PERMISSIONMANAGER_CONTRACTID);
	NS_ENSURE_SUCCESS(rv, FALSE);
	NS_ENSURE_TRUE(pman, FALSE);
	
	miniCOMPtr<nsIURI> uri;
	if (! new_uri(url, uri)) {
		return FALSE;
	}
	NS_ENSURE_TRUE(uri, FALSE);
	
	/* 0 is reserved by gecko, so this uses 0 to tell remove. */
	if (perm == 0) {
		nsEmbedCString hoststr;
		rv = uri->GetHost(hoststr);
		NS_ENSURE_SUCCESS(rv, FALSE);
		
		rv = pman->Remove(hoststr, type);
	}
	else {
		rv = pman->Add(uri, type, perm);
	}
	
	NS_ENSURE_SUCCESS(rv, FALSE);
	return TRUE;
}


void utils_HostAllow(char *url, char *type)
{
	addperm(url, type, nsIPermissionManager::ALLOW_ACTION);
}

void utils_HostDeny(char *url, char *type)
{
	addperm(url, type, nsIPermissionManager::DENY_ACTION);
}

void utils_HostRemove(char *url, char *type)
{
	addperm(url, type, 0);
}
